@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: mgd77manage.c,v 1.91 2007/10/07 07:10:41 guru Exp $
+ *	$Id: mgd77manage.c,v 1.103 2008/04/26 00:38:50 guru Exp $
  *
- *    Copyright (c) 2005-2007 by P. Wessel
+ *    Copyright (c) 2005-2008 by P. Wessel
  * mgd77manage is used to (1) remove data columns from mgd77+ files
  * or (2) add a new data column to mgd77+ files.  Data can be added
  * from data tables, created from reference field formulas, or by
@@ -32,6 +32,7 @@
 #define ADD_IGRF	2
 #define ADD_CARTER	3
 #define ADD_GRAV	4
+#define ADD_RMAG	5
 
 #define N_E77_MODES	5
 #define E77_HEADER_MODE	0
@@ -40,18 +41,18 @@
 #define E77_VALUES_MODE	3
 #define E77_SLOPES_MODE	4
 
-#define NCPOS_TIME	0
-#define NCPOS_LAT	3
-#define NCPOS_LON	4
-
 #define set_bit(k) (1 << (k))
 
 int main (int argc, char **argv)
 {
-	int i, j, k = 0, ii, jj, n = 0, argno, n_cruises = 0, n_paths = 0, ij, mx = 0, my, pos, column, result;
-	int width, n_delete = 0, GF_version = MGD77_NOT_SET, n_expected_fields, n_fields = 0, n_sampled = 0, n_changed = 0;
-	int kind = GMT_IS_FLOAT, dims[2], set, var_id, n_dims = 0, dist_flag = 2, n_bad, e77_mode[N_E77_MODES];
-	int interpolant = BCR_BICUBIC;
+	int var_id, n_dims = 0, dims[2];
+	
+	int i, j, k = 0, ii, jj, argno, n_cruises = 0, n_paths = 0, mx = 0, my, pos, column, result;
+	int width, n_delete = 0, GF_version = MGD77_NOT_SET, n_expected_fields, n_fields = 0;
+	int kind = GMT_IS_FLOAT, MTF_col = 1, set, dist_flag = 2, n_bad, e77_mode[N_E77_MODES];
+	int interpolant = BCR_BICUBIC, check;
+	
+	GMT_LONG ij, n_sampled = 0, n_changed = 0, n = 0;
 	
 	size_t n_alloc = GMT_CHUNK, start[2] = {0, 0}, count[2] = {0, 0};
 	
@@ -267,6 +268,8 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t      If filename is - we read from stdin.\n");
 		fprintf (stderr, "\t   c: Create a new column to be calculated from existing columns.  Add code:\n");
 		fprintf (stderr, "\t        m = IGRF total field, c = Carter correction, g = IGF (\"normal gravity\")\n");
+		fprintf (stderr, "\t        r = recoputed magnetic anomaly rmag = mtfx - IGRF total field\n");
+		fprintf (stderr, "\t        Append x for which mtfx field to use (1 or 2) [1]\n");
 		fprintf (stderr, "\t        For g, optionally append 1-4 to select the gravity formula to use:\n");
 		fprintf (stderr, "\t        1 = Heiskanen 1924, 2 = International 1930, 3 = IGF1967, 4 = IGF1980.\n");
 		fprintf (stderr, "\t        [Default uses formula specified in the MGD77 header, or 4 if not valid]\n");
@@ -334,7 +337,7 @@ int main (int argc, char **argv)
 		exit (EXIT_FAILURE);
 	}
 
-	got_table = (got_a || got_d || got_e || got_n || got_t);	/* Got a table to read */
+	got_table = (got_a || got_d || got_n || got_t);	/* Got a table to read */
 	got_grid = (got_g || got_i);					/* Got a grid to read */
 	c_nc_type = (nc_type) irint (parameters[COL_TYPE]);		/* NC data type */
 	strings = (c_nc_type == NC_CHAR);				/* TRUE if our new column contains strings */
@@ -386,12 +389,12 @@ int main (int argc, char **argv)
 	}
 	MGD77_Set_Unit (d_unit, &dist_scale, -1);	/* Gets scale which multiplies meters to chosen distance unit */
 
-	memset ((void *)not_given, (int)no_char, GMT_TEXT_LEN);	/* Text representing "no text value" */
+	memset ((void *)not_given, (int)no_char, (size_t)GMT_TEXT_LEN);	/* Text representing "no text value" */
 	not_given[GMT_TEXT_LEN-1] = '\0';
 	fp_err = (In.verbose_dest == 1) ? GMT_stdout : stderr;
 	
 	if (got_c) {	/* Calculate values to be stored */
-		int version = MGD77_NOT_SET;
+		int version = MGD77_NOT_SET, mfield = 1;
 		/* "file" is here either m, c, or g[1-4] */
 		if (file[0] == 'm' && file[1] == '\0') {
 			got_c = ADD_IGRF;
@@ -404,6 +407,10 @@ int main (int argc, char **argv)
 			got_c = ADD_GRAV;
 			GF_version = version;
 		}
+		else if (file[0] == 'r' && (file[1] == '\0' || ((mfield = (file[1] - '0')) >= 1 && mfield <= 2)) ) {
+			got_c = ADD_RMAG;
+			MTF_col = mfield;
+		}
 		else {
 			fprintf(stderr, "%s: ERROR: -Ac expects m, c, or g[1-4]\n", GMT_program);
 			exit (EXIT_FAILURE);
@@ -451,8 +458,9 @@ int main (int argc, char **argv)
 		/* Select plain Mercator on a sphere with -Jm1 -R0/360/-72.006/+72.006 */
 		gmtdefs.ellipsoid = GMT_N_ELLIPSOIDS - 1;
 		project_info.units_pr_degree = TRUE;
-		project_info.m_got_parallel = FALSE;
-		project_info.pars[0] = 1.0;
+		project_info.pars[0] = 180.0;
+		project_info.pars[1] = 0.0;
+		project_info.pars[2] = 1.0;
 		project_info.projection = GMT_MERCATOR;
 		
 		GMT_err_fail (GMT_map_setup (0.0, 360.0, GMT_IMG_MINLAT, GMT_IMG_MAXLAT), "");
@@ -489,7 +497,7 @@ int main (int argc, char **argv)
 		i2 = (short int *) GMT_memory (VNULL, (size_t)(grd.nx), sizeof (short int), GMT_program);
 		for (j = 0; j < grd.ny; j++) {	/* Read all the rows, offset by 2 boundary rows and cols */
 			ij = (j + GMT_pad[3]) * mx + GMT_pad[0];
-			fread ((void *)i2, sizeof (short int), grd.nx, fp);
+			fread ((void *)i2, sizeof (short int), (size_t)grd.nx, fp);
 			for (i = 0; i < grd.nx; i++) {	/* Process this row's values */
 				switch (track_decode) {
 					case 0:	/* No encoded track flags, do nothing */
@@ -563,7 +571,7 @@ int main (int argc, char **argv)
 			if ((GMT_io.status & GMT_IO_EOF)) continue;	/* At EOF */
 
 			if (GMT_io.status & GMT_IO_MISMATCH) {
-				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields, n_expected_fields, n);
+				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, n);
 				exit (EXIT_FAILURE);
 			}
 
@@ -580,8 +588,8 @@ int main (int argc, char **argv)
 			else
 				colvalue[n] = in[0];
 			n++;
-			if (n == (int)n_alloc) {
-				n_alloc += GMT_CHUNK;
+			if (n == (GMT_LONG)n_alloc) {
+				n_alloc <<= 1;
 				if (strings)
 					tmp_string = (char **) GMT_memory ((void *)tmp_string, n_alloc, sizeof (char *), GMT_program);
 				else
@@ -694,7 +702,7 @@ int main (int argc, char **argv)
 			for (i = 0; i < k; i++) if (history[i] == '\n') history[i] = ' ';	/* Remove the \n returned by ctime() */
 			history[k++] = '\n';	history[k] = '\0';				/* Add LF at end of line */
 			k += strlen (D->H.history);
-			D->H.history = (char *)GMT_memory ((void *)D->H.history, k, sizeof (char), GMT_program);
+			D->H.history = (char *)GMT_memory ((void *)D->H.history, (size_t)k, sizeof (char), GMT_program);
 			strcat (D->H.history, history);		/* MGD77_Write_FILE_cdf will use this to create the history attribute, thus preserving earlier history */
 
 			if (MGD77_Write_File (In.path, &In, D)) {	/* Create the new, slimmer file */
@@ -734,7 +742,7 @@ int main (int argc, char **argv)
 			xvar = (double *)D->values[ix];
 			yvar = (double *)D->values[iy];
 			tvar = (double *)D->values[it];
-			colvalue = (double *)GMT_memory (VNULL, D->H.n_records, sizeof (double), GMT_program);
+			colvalue = (double *)GMT_memory (VNULL, (size_t)D->H.n_records, sizeof (double), GMT_program);
 			
 			for (i = n_sampled = 0; i < D->H.n_records; i++) {
 				GMT_gcal_from_dt (tvar[i], &cal);			/* No adjust for TZ; this is GMT UTC time */
@@ -744,7 +752,7 @@ int main (int argc, char **argv)
 				colvalue[i] = (MGD77_igrf10syn (0, date, 1, 0.0, xvar[i], yvar[i], IGRF)) ? GMT_d_NaN : IGRF[MGD77_IGRF_F];
 				n_sampled++;
 			}
-			if (gmtdefs.verbose) fprintf (stderr, "%s: Estimated IGRF at %d locations out of %d for cruise %s\n", GMT_program, n_sampled, D->H.n_records, list[argno]);
+			if (gmtdefs.verbose) fprintf (stderr, "%s: Estimated IGRF at %ld locations out of %ld for cruise %s\n", GMT_program, n_sampled, D->H.n_records, list[argno]);
 		}
 		else if (got_c == ADD_GRAV) {	/* Append IGF column */
 			int ix, iy;
@@ -762,10 +770,10 @@ int main (int argc, char **argv)
 			}
 			xvar = (double *)D->values[ix];
 			yvar = (double *)D->values[iy];
-			colvalue = (double *)GMT_memory (VNULL, D->H.n_records, sizeof (double), GMT_program);
+			colvalue = (double *)GMT_memory (VNULL, (size_t)D->H.n_records, sizeof (double), GMT_program);
 			
 			for (i = 0; i < D->H.n_records; i++) colvalue[i] = MGD77_Theoretical_Gravity (xvar[i], yvar[i], GF_version);
-			if (gmtdefs.verbose) fprintf (stderr, "%s: Estimated IGRF at %d locations out of %d for cruise %s\n", GMT_program, D->H.n_records, D->H.n_records, list[argno]);
+			if (gmtdefs.verbose) fprintf (stderr, "%s: Estimated IGRF at %ld locations out of %ld for cruise %s\n", GMT_program, D->H.n_records, D->H.n_records, list[argno]);
 		}
 		else if (got_c == ADD_CARTER) {	/* Append Carter correction column */
 			int ix, iy, it;
@@ -778,10 +786,38 @@ int main (int argc, char **argv)
 			xvar = (double *)D->values[ix];
 			yvar = (double *)D->values[iy];
 			tvar = (double *)D->values[it];
-			colvalue = (double *)GMT_memory (VNULL, D->H.n_records, sizeof (double), GMT_program);
+			colvalue = (double *)GMT_memory (VNULL, (size_t)D->H.n_records, sizeof (double), GMT_program);
 			
 			for (i = 0; i < D->H.n_records; i++) colvalue[i] = MGD77_carter_correction (xvar[i], yvar[i], 1000.0 * tvar[i], &Carter);
-			if (gmtdefs.verbose) fprintf (stderr, "%s: Estimated IGRF at %d locations out of %d for cruise %s\n", GMT_program, D->H.n_records, D->H.n_records, list[argno]);
+			if (gmtdefs.verbose) fprintf (stderr, "%s: Estimated IGRF at %ld locations out of %ld for cruise %s\n", GMT_program, D->H.n_records, D->H.n_records, list[argno]);
+		}
+		else if (got_c == ADD_RMAG) {	/* Append recomputed residual mag column */
+			int ix, iy, it, im;
+			double n_days, date, *xvar, *yvar, *tvar, *mvar, IGRF[7];
+			char field[5];
+			
+			if ((ix = skip_if_missing ("lon",  list[argno], &In, D)) == MGD77_NOT_SET) continue;
+			if ((iy = skip_if_missing ("lat",  list[argno], &In, D)) == MGD77_NOT_SET) continue;
+			if ((it = skip_if_missing ("time", list[argno], &In, D)) == MGD77_NOT_SET) continue;
+			sprintf (field, "mtf%d", MTF_col);
+			if ((im = skip_if_missing (field, list[argno], &In, D)) == MGD77_NOT_SET) continue;
+
+			xvar = (double *)D->values[ix];
+			yvar = (double *)D->values[iy];
+			tvar = (double *)D->values[it];
+			mvar = (double *)D->values[im];
+			colvalue = (double *)GMT_memory (VNULL, (size_t)D->H.n_records, sizeof (double), GMT_program);
+			
+			for (i = n_sampled = 0; i < D->H.n_records; i++) {
+				GMT_gcal_from_dt (tvar[i], &cal);			/* No adjust for TZ; this is GMT UTC time */
+				n_days = (GMT_is_gleap (cal.year)) ? 366.0 : 365.0;	/* Number of days in this year */
+				/* Get date as decimal year */
+				date = cal.year + cal.day_y / n_days + (cal.hour * GMT_HR2SEC_I + cal.min * GMT_MIN2SEC_I + cal.sec) * GMT_SEC2DAY;
+				check = MGD77_igrf10syn (0, date, 1, 0.0, xvar[i], yvar[i], IGRF);
+				colvalue[i] = (check) ? GMT_d_NaN : mvar[i] - IGRF[MGD77_IGRF_F];
+				n_sampled++;
+			}
+			if (gmtdefs.verbose) fprintf (stderr, "%s: Estimated recomputed magnetic anomaly at %ld locations out of %ld for cruise %s\n", GMT_program, n_sampled, D->H.n_records, list[argno]);
 		}
 		else if (got_grid) {	/* Sample grid along track (or Mercator-projected) track */
 			int ix, iy;
@@ -792,7 +828,7 @@ int main (int argc, char **argv)
 
 			xvar = (double *)D->values[ix];
 			yvar = (double *)D->values[iy];
-			colvalue = (double *)GMT_memory (VNULL, D->H.n_records, sizeof (double), GMT_program);
+			colvalue = (double *)GMT_memory (VNULL, (size_t)D->H.n_records, sizeof (double), GMT_program);
 			
 			for (i = n_sampled = 0; i < D->H.n_records; i++) {
 				colvalue[i] = GMT_d_NaN;	/* In case we are outside grid */
@@ -823,14 +859,14 @@ int main (int argc, char **argv)
 				}
 				n_sampled++;
 			}
-			if (gmtdefs.verbose) fprintf (stderr, "%s: Sampled grid at %d locations out of %d for cruise %s\n", GMT_program, n_sampled, D->H.n_records, list[argno]);
+			if (gmtdefs.verbose) fprintf (stderr, "%s: Sampled grid at %ld locations out of %ld for cruise %s\n", GMT_program, n_sampled, D->H.n_records, list[argno]);
 		}
 		else if (got_a) {	/* Just got a single column to paste in, assuming the row numbers match */
 			if (n != D->H.n_records) {
-				fprintf (stderr, "%s: Extra column data records (%d) do not match # of cruise records (%d) for %s\n", GMT_program, n, D->H.n_records, list[argno]);
+				fprintf (stderr, "%s: Extra column data records (%ld) do not match # of cruise records (%ld) for %s\n", GMT_program, n, D->H.n_records, list[argno]);
 				exit (EXIT_FAILURE);
 			}
-			if (gmtdefs.verbose) fprintf (stderr, "%s: Appended column data for all %d records for cruise %s\n", GMT_program, D->H.n_records, list[argno]);
+			if (gmtdefs.verbose) fprintf (stderr, "%s: Appended column data for all %ld records for cruise %s\n", GMT_program, D->H.n_records, list[argno]);
 		}
 		else if (got_d || got_n || got_t) {	/* Got either (time,data) or (dist,data) */
 			int ix, iy, it;
@@ -859,24 +895,24 @@ int main (int argc, char **argv)
 				GMT_free ((void *)y);
 			}
 			else if (strings && n < D->H.n_records) {	/* Only update the exact matching records */
-				text = (char *) GMT_memory (VNULL, D->H.n_records * LEN, sizeof (char), GMT_program);
+				text = (char *) GMT_memory (VNULL, (size_t)(D->H.n_records * LEN), sizeof (char), GMT_program);
 				for (i = j = n_sampled = 0; i < D->H.n_records && j < n; i++) {
 					match_value = (got_n) ? i+1 : x[i];
-					strncpy (&text[i*LEN], not_given, (int)LEN);	/* In case we have no data at this time */
+					strncpy (&text[i*LEN], not_given, (size_t)LEN);	/* In case we have no data at this time */
 					while (coldnt[j] < match_value && j < n) j++;
 					if (coldnt[j] == match_value) {
-						strncpy (&text[i*LEN], tmp_string[j], (int)LEN);
+						strncpy (&text[i*LEN], tmp_string[j], (size_t)LEN);
 						n_sampled++;
 					}
 				}
 				GMT_free ((void *)tmp_string);
-				if (gmtdefs.verbose) fprintf (stderr, "%s: Appended column data for %d locations out of %d for cruise %s\n", GMT_program, n_sampled, D->H.n_records, list[argno]);
+				if (gmtdefs.verbose) fprintf (stderr, "%s: Appended column data for %ld locations out of %ld for cruise %s\n", GMT_program, n_sampled, D->H.n_records, list[argno]);
 			}
 			else if (strings) {	/* One to one match */
-				text = (char *) GMT_memory (VNULL, D->H.n_records * LEN, sizeof (char), GMT_program);
-				for (i = 0; i < n; i++) strncpy (&text[i*LEN], tmp_string[i], (int)LEN);
+				text = (char *) GMT_memory (VNULL, (size_t)(D->H.n_records * LEN), sizeof (char), GMT_program);
+				for (i = 0; i < n; i++) strncpy (&text[i*LEN], tmp_string[i], (size_t)LEN);
 				GMT_free ((void *)tmp_string);
-				if (gmtdefs.verbose) fprintf (stderr, "%s: Appended column data for %d locations out of %d for cruise %s\n", GMT_program, n_sampled, D->H.n_records, list[argno]);
+				if (gmtdefs.verbose) fprintf (stderr, "%s: Appended column data for %ld locations out of %ld for cruise %s\n", GMT_program, n_sampled, D->H.n_records, list[argno]);
 			}
 			else {	/* Only update the exact matching records */
 				y = (double *) GMT_memory (VNULL, (size_t)D->H.n_records, sizeof (double), GMT_program);
@@ -890,7 +926,7 @@ int main (int argc, char **argv)
 					}
 				}
 				memcpy ((void *)colvalue, (void *)y, (size_t)(D->H.n_records * sizeof (double)));
-				if (gmtdefs.verbose) fprintf (stderr, "%s: Appended column data for %d locations out of %d for cruise %s\n", GMT_program, n_sampled, D->H.n_records, list[argno]);
+				if (gmtdefs.verbose) fprintf (stderr, "%s: Appended column data for %ld locations out of %ld for cruise %s\n", GMT_program, n_sampled, D->H.n_records, list[argno]);
 				GMT_free ((void *)y);
 			}
 			if (got_d) GMT_free ((void *)d);
@@ -901,10 +937,13 @@ int main (int argc, char **argv)
 			 * Decode error flags to give bit flag, store in colvalue
 			 */
 			FILE *fp_e;
+			int var_id, item;
 			char ID[16], date[16], field[GMT_TEXT_LEN], efile[BUFSIZ], E77[256], timestamp[GMT_TEXT_LEN], answer[BUFSIZ], code[BUFSIZ], YorN, kind;
-			int n_recs, type, number, item, it, rec, id, var_id, key, length, n_E77_flags, from, to, day, month, year;
+			GMT_LONG n_recs, rec, number;
+			int type, it, id, key, n_E77_flags, from, to, day, month, year;
 			int n_E77_headers, n_E77_scales, n_E77_offsets, n_unprocessed, e_error = 0;
 			unsigned int *flags, pattern;
+			size_t length;
 			BOOLEAN has_time, old_flags;
 			struct MGD77_HEADER_PARAMS *P;
 			double rec_time, del_t, value, *tvar = VNULL;
@@ -934,7 +973,7 @@ int main (int argc, char **argv)
 				fprintf(stderr, "%s: ERROR: Could not read record #1 from %s.e77 - aborting\n", GMT_program, list[argno]);
 				e_error++;
 			}
-			sscanf (&line[1], "%*s %s %*s %*s %*s %*s %*s %s %*s %d", ID, date, &n_recs);
+			sscanf (&line[1], "%*s %s %*s %*s %*s %*s %*s %s %*s %ld", ID, date, &n_recs);
 			if (strcmp (In.NGDC_id, ID)) {
 				fprintf(stderr, "%s: ERROR: E77 Conflict %s : ID = %s versus %s - aborting\n", GMT_program, efile, ID, In.NGDC_id);
 				e_error++;
@@ -952,14 +991,14 @@ int main (int argc, char **argv)
 				e_error++;
 			}
 			if (n_recs != D->H.n_records) {
-				fprintf(stderr, "%s: ERROR: E77 Conflict %s: n_recs = %d versus %d = aborting\n", GMT_program, efile, n_recs, D->H.n_records);
+				fprintf(stderr, "%s: ERROR: E77 Conflict %s: n_recs = %ld versus %ld = aborting\n", GMT_program, efile, n_recs, D->H.n_records);
 				e_error++;
 			}
 			verified = FALSE;
-			while (fgets (line, BUFSIZ, fp_e) && strncmp (line, "# Errata: Header", 14)) {	/* Read until we get to Header record section */
+			while (fgets (line, BUFSIZ, fp_e) && strncmp (line, "# Errata: Header", (size_t)14)) {	/* Read until we get to Header record section */
 				if (line[0] == '#') continue;	/* Skip comments */
 				GMT_chop (line);		/* Rid the world of CR/LF */
-				if (!strncmp (line, "Y Errata table verification status", 34)) verified = TRUE;
+				if (!strncmp (line, "Y Errata table verification status", (size_t)34)) verified = TRUE;
 			}
 			if (!verified) {
 				fprintf(stderr, "%s: ERROR: E77 file %s not yet verified.  E77 not applied\n", GMT_program, efile);
@@ -988,7 +1027,7 @@ int main (int argc, char **argv)
 					sscanf (line, "%c-%c-%[^-]-%[^-]-%d", &YorN, &kind, ID, field, &item);
 				}
 				else				/* Data record */
-					sscanf (line, "%c %s %s %d %s", &YorN, ID, timestamp, &rec, code);
+					sscanf (line, "%c %s %s %ld %s", &YorN, ID, timestamp, &rec, code);
 				if (strcmp (In.NGDC_id, ID)) {
 					fprintf(stderr, "%s: ERROR: E77 Conflict %s : ID = %s versus %s in header records!\n", GMT_program, efile, ID, In.NGDC_id);
 					e_error++;
@@ -1009,15 +1048,15 @@ int main (int argc, char **argv)
 			/* OK, here we believe the E77 file contains the correct information for this cruise. Rewind and start from top */
 			
 			rewind (fp_e);
-			while (fgets (line, BUFSIZ, fp_e) && strncmp (line, "# Errata: Header", 14));	/* Read until we get to Header record section */
+			while (fgets (line, BUFSIZ, fp_e) && strncmp (line, "# Errata: Header", (size_t)14));	/* Read until we get to Header record section */
 			
-			flags = (unsigned int *)GMT_memory (VNULL, D->H.n_records, sizeof (unsigned int), GMT_program);
+			flags = (unsigned int *)GMT_memory (VNULL, (size_t)D->H.n_records, sizeof (unsigned int), GMT_program);
 			n_E77_flags = n_E77_headers = n_E77_scales = n_E77_offsets = 0;
 
 			MGD77_nc_status (nc_open (In.path, NC_WRITE, &In.nc_id));	/* Open the file */
 			MGD77_nc_status (nc_redef (In.nc_id));				/* Enter define mode */
 			old_flags = MGD77_Remove_E77 (&In);				/* Remove any previously revised header parameters */
-			while (fgets (line, BUFSIZ, fp_e) && strncmp (line, "# Errata: Data", 14)) {	/* Read until we get to data record section */
+			while (fgets (line, BUFSIZ, fp_e) && strncmp (line, "# Errata: Data", (size_t)14)) {	/* Read until we get to data record section */
 				if (line[0] == '#' || line[0] == '\n') continue;	/* Skip comments */
 				GMT_chop (line);					/* Rid the world of CR/LF */
 				/* Example of expected line:  
@@ -1031,11 +1070,11 @@ int main (int argc, char **argv)
 				}
 				if (field[0] == 'H') {
 					type = 0;
-					number = atoi (&field[1]);
+					number = (GMT_LONG)atoi (&field[1]);
 				}
 				else {
 					type = 1;
-					number = item;
+					number = (GMT_LONG)item;
 				}
 				if (kind == 'W') {	/* Output the warning (if Y) and goto next line*/
 					if (YorN == 'Y') fprintf (stderr, "%s: WARNING: %s\n", list[argno], line);
@@ -1053,23 +1092,23 @@ int main (int argc, char **argv)
 					
 					switch (key) {
 						case MGD77_BAD_HEADER_RECNO:
-							fprintf (stderr, "%s: Warning: Sequence number %d is outside range - skipped\n", GMT_program, number);
+							fprintf (stderr, "%s: Warning: Sequence number %ld is outside range - skipped\n", GMT_program, number);
 							break;
 						case MGD77_BAD_HEADER_ITEM:
-							fprintf (stderr, "%s: Warning: Sequence number %d, Item %d is not supported - skipped\n", GMT_program, number, item);
+							fprintf (stderr, "%s: Warning: Sequence number %ld, Item %d is not supported - skipped\n", GMT_program, number, item);
 							break;
 						default:	/* Update internal structure as well as netCDF file attributes */
 							if (key == (MGD77_N_HEADER_PARAMS-1)) {	/* Could be any of records 18-24 */
 								char att[32];
 								length = strlen (answer);
 								item = number - 17;
-								strncpy (P->Additional_Documentation[item], answer, length);
+								strncpy (P->Additional_Documentation[item], answer, (size_t)length);
 								sprintf (att, "%s_%d", MGD77_Header_Lookup[key].name, item);
 								MGD77_Put_Param (&In, att, length, P->Additional_Documentation[item]);
 							}
 							else {
 								length = (MGD77_Header_Lookup[key].length == 1) ? 1 : strlen (answer);
-								strncpy (MGD77_Header_Lookup[key].ptr, answer, length);
+								strncpy (MGD77_Header_Lookup[key].ptr, answer, (size_t)length);
 								MGD77_Put_Param (&In, MGD77_Header_Lookup[key].name, length, MGD77_Header_Lookup[key].ptr);
 							}
 							n_E77_headers++;
@@ -1087,14 +1126,14 @@ int main (int argc, char **argv)
 							case E77_HDR_SCALE:	/* Correction scale factor */
 								if (D->H.info[set].col[id].corr_factor == 1.0) {	/* Must add a new attribute to the file */
 									D->H.info[set].col[id].corr_factor = value;
-									MGD77_nc_status (nc_put_att_double (In.nc_id, D->H.info[set].col[id].var_id, "corr_factor", NC_DOUBLE, 1, &D->H.info[set].col[id].corr_factor));
+									MGD77_nc_status (nc_put_att_double (In.nc_id, D->H.info[set].col[id].var_id, "corr_factor", NC_DOUBLE, (size_t)1, &D->H.info[set].col[id].corr_factor));
 								}
 								n_E77_scales++;
 								break;
 							case E77_HDR_DCSHIFT:	/* Correction offset */
 								if (D->H.info[set].col[id].corr_offset == 0.0) {	/* Must add a new attribute to the file */
 									D->H.info[set].col[id].corr_offset = value;
-									MGD77_nc_status (nc_put_att_double (In.nc_id, D->H.info[set].col[id].var_id, "corr_offset", NC_DOUBLE, 1, &D->H.info[set].col[id].corr_offset));
+									MGD77_nc_status (nc_put_att_double (In.nc_id, D->H.info[set].col[id].var_id, "corr_offset", NC_DOUBLE, (size_t)1, &D->H.info[set].col[id].corr_offset));
 								}
 								n_E77_offsets++;
 								break;
@@ -1120,7 +1159,7 @@ int main (int argc, char **argv)
 			else
 				tvar = (double *)D->values[it];
 			while (fgets (line, BUFSIZ, fp_e)) {	/* Read until EOF */
-				sscanf (line, "%c %s %s %d %s", &YorN, ID, timestamp, &rec, code);
+				sscanf (line, "%c %s %s %ld %s", &YorN, ID, timestamp, &rec, code);
 				if (strcmp (In.NGDC_id, ID)) {
 					fprintf(stderr, "%s: ERROR: E77 Conflict %s : ID = %s versus %s in data records - skipped\n", GMT_program, efile, ID, In.NGDC_id);
 					e_error++;
@@ -1135,8 +1174,11 @@ int main (int argc, char **argv)
 				rec--;	/* E77 starts with rec = 1 for first data record */
 				if (has_time) {
 					if (GMT_verify_expectations (GMT_IS_ABSTIME, GMT_scanf (timestamp, GMT_IS_ABSTIME, &rec_time), timestamp)) {
-						fprintf (stderr, "%s: ERROR: E77 time stamp (%s) in wrong format? - skipped\n", GMT_program, timestamp);
-						continue;
+						if (!strcmp(timestamp,"NaN")) fprintf (stderr, "%s: WARNING: E77 time stamp %s, using recno\n", GMT_program, timestamp);
+						else {
+							fprintf (stderr, "%s: ERROR: E77 time stamp (%s) in wrong format? - skipped\n", GMT_program, timestamp);
+							continue;
+						}
 					}
 					del_t = fabs (tvar[rec] - rec_time);
 					if (del_t > (0.06 + GMT_CONV_LIMIT)) {	/* 0.06 is finest time step in MGD77 file so we allow that much slop */
@@ -1172,12 +1214,10 @@ int main (int argc, char **argv)
 									flags[rec] |= set_bit(NCPOS_LAT);
 									n_E77_flags++;
 									break;
-								case 'E':	/* Undefined lat */
-									flags[rec] |= set_bit(NCPOS_LAT);
-									n_E77_flags++;
-									break;
-								case 'F':	/* Undefined lon */
+								case 'E':	/* Undefined nav - flag time, lon, lat */
+									flags[rec] |= set_bit(NCPOS_TIME);
 									flags[rec] |= set_bit(NCPOS_LON);
+									flags[rec] |= set_bit(NCPOS_LAT);
 									n_E77_flags++;
 									break;
 								default:
@@ -1214,9 +1254,9 @@ int main (int argc, char **argv)
 			sprintf (E77, "%s [%s] E77 corrections applied to header: %d scale: %d offset: %d flags: %d", ctime(&now), In.user, n_E77_headers, n_E77_scales, n_E77_offsets, n_E77_flags);
 			for (i = 0; E77[i]; i++) if (E77[i] == '\n') E77[i] = ' ';	/* Remove the \n returned by ctime() */
 			k = strlen (E77);
-			D->H.E77 = (char *)GMT_memory ((void *)D->H.E77, k+1, sizeof (char), GMT_program);
+			D->H.E77 = (char *)GMT_memory ((void *)D->H.E77, (size_t)(k+1), sizeof (char), GMT_program);
 			strcpy (D->H.E77, E77);
-			MGD77_nc_status (nc_put_att_text (In.nc_id, NC_GLOBAL, "E77", k, D->H.E77));
+			MGD77_nc_status (nc_put_att_text (In.nc_id, NC_GLOBAL, "E77", (size_t)k, D->H.E77));
 		
 			old_flags =  (nc_inq_varid (In.nc_id, "MGD77_flags", &var_id) == NC_NOERR);	/* TRUE if flag variable exists already */
 			
@@ -1226,7 +1266,7 @@ int main (int argc, char **argv)
 				else {	/* We need to define the flags for the first time */
 					dims[0] = In.nc_recid;
 					MGD77_nc_status (nc_def_var (In.nc_id, "MGD77_flags", NC_INT, 1, dims, &var_id));	/* Define an array variable */
-					memset ((void *)answer, 0, BUFSIZ);	/* No default answer */
+					memset ((void *)answer, 0, (size_t)BUFSIZ);	/* No default answer */
 					strcpy (answer, "MGD77 flags (ON = Bad, OFF = Good) derived from E77 errata");
 					MGD77_nc_status (nc_put_att_text (In.nc_id, var_id, "comment", strlen (answer), answer));
 					D->flags[0] = flags;
@@ -1261,7 +1301,7 @@ int main (int argc, char **argv)
 			error = 0;
 			if (LEN) {
 				if (OLDLEN != LEN) {
-					fprintf (stderr, "%s: Revised text column %s differs in width (%d) from the old values (%d.\n", GMT_program, c_abbrev, (int)LEN, (int)OLDLEN);
+					fprintf (stderr, "%s: Revised text column %s differs in width (%d) from the old values (%d).\n", GMT_program, c_abbrev, (int)LEN, (int)OLDLEN);
 					error = TRUE;
 				}
 				if (constant && n_dims == 2) {
@@ -1319,12 +1359,12 @@ int main (int argc, char **argv)
 		
 		if (c_name[0]) MGD77_nc_status (nc_put_att_text   (In.nc_id, var_id, "long_name", strlen (c_name), c_name));
 		if (c_units[0]) MGD77_nc_status (nc_put_att_text   (In.nc_id, var_id, "units", strlen (c_units), c_units));
-		MGD77_nc_status (nc_put_att_double   (In.nc_id, var_id, "actual_range", NC_DOUBLE, 2, limits));
+		MGD77_nc_status (nc_put_att_double   (In.nc_id, var_id, "actual_range", NC_DOUBLE, (size_t)2, limits));
 		if (c_comment[0]) MGD77_nc_status (nc_put_att_text   (In.nc_id, var_id, "comment", strlen (c_comment), c_comment));
-		MGD77_nc_status (nc_put_att_double (In.nc_id, var_id, "_FillValue", c_nc_type, 1, &MGD77_NaN_val[c_nc_type]));
-		MGD77_nc_status (nc_put_att_double (In.nc_id, var_id, "missing_value", c_nc_type, 1, &MGD77_NaN_val[c_nc_type]));
-		if (parameters[COL_SCALE]  != 1.0) MGD77_nc_status (nc_put_att_double (In.nc_id, var_id, "scale_factor", NC_DOUBLE, 1, &parameters[COL_SCALE]));
-		if (parameters[COL_OFFSET] != 0.0) MGD77_nc_status (nc_put_att_double (In.nc_id, var_id, "add_offset",   NC_DOUBLE, 1, &parameters[COL_OFFSET]));
+		MGD77_nc_status (nc_put_att_double (In.nc_id, var_id, "_FillValue", c_nc_type, (size_t)1, &MGD77_NaN_val[c_nc_type]));
+		MGD77_nc_status (nc_put_att_double (In.nc_id, var_id, "missing_value", c_nc_type, (size_t)1, &MGD77_NaN_val[c_nc_type]));
+		if (parameters[COL_SCALE]  != 1.0) MGD77_nc_status (nc_put_att_double (In.nc_id, var_id, "scale_factor", NC_DOUBLE, (size_t)1, &parameters[COL_SCALE]));
+		if (parameters[COL_OFFSET] != 0.0) MGD77_nc_status (nc_put_att_double (In.nc_id, var_id, "add_offset",   NC_DOUBLE, (size_t)1, &parameters[COL_OFFSET]));
 					
 		/* Update history */
 
@@ -1334,7 +1374,7 @@ int main (int argc, char **argv)
 		for (i = 0; i < k; i++) if (history[i] == '\n') history[i] = ' ';	/* Remove the \n returned by ctime() */
 		history[k++] = '\n';	history[k] = '\0';				/* Add LF at end of line */
 		k += strlen (D->H.history);
-		D->H.history = (char *)GMT_memory ((void *)D->H.history, k, sizeof (char), GMT_program);
+		D->H.history = (char *)GMT_memory ((void *)D->H.history, (size_t)k, sizeof (char), GMT_program);
 		strcat (D->H.history, history);
 		MGD77_nc_status (nc_put_att_text (In.nc_id, NC_GLOBAL, "history", strlen (D->H.history), D->H.history));
 		
@@ -1346,7 +1386,7 @@ int main (int argc, char **argv)
 			if (LEN)
 				MGD77_nc_status (nc_put_vara_schar (In.nc_id, var_id, start, &count[1], (signed char *)text));	/* Just write one text string */
 			else {
-				n_bad = MGD77_do_scale_offset_before_write (&single_val, colvalue, 1, parameters[COL_SCALE], parameters[COL_OFFSET], c_nc_type);
+				n_bad = MGD77_do_scale_offset_before_write (&single_val, colvalue, (size_t)1, parameters[COL_SCALE], parameters[COL_OFFSET], c_nc_type);
 				MGD77_nc_status (nc_put_var1_double (In.nc_id, var_id, start, &single_val));
 			}
 		}
@@ -1378,13 +1418,16 @@ int main (int argc, char **argv)
 	
 	if (gmtdefs.verbose) {
 		if (delete)
-			fprintf (stderr, "%s: Removed %d data columns from %d MGD77 files\n", GMT_program, n_delete, n_changed);
+			fprintf (stderr, "%s: Removed %d data columns from %ld MGD77 files\n", GMT_program, n_delete, n_changed);
 		else if (got_e)
-			fprintf (stderr, "%s: E77 corrections applied to %d MGD77 files\n", GMT_program, n_changed);
+			fprintf (stderr, "%s: E77 corrections applied to %ld MGD77 files\n", GMT_program, n_changed);
 		else
-			fprintf (stderr, "%s: Sampled data for %d MGD77 files\n", GMT_program, n_changed);
+			fprintf (stderr, "%s: Sampled data for %ld MGD77 files\n", GMT_program, n_changed);
 	}
 	
+	MGD77_Path_Free (n_paths, list);
+	MGD77_end (&In);
+
 	exit (EXIT_SUCCESS);
 }
 
@@ -1497,10 +1540,10 @@ int got_default_answer (char *line, char *answer)
 	int i, k, len;
 	
 	len = strlen(line) - 1;
-	memset ((void *)answer, 0, BUFSIZ);	/* No default answer */
+	memset ((void *)answer, 0, (size_t)BUFSIZ);	/* No default answer */
 	if (line[len] == ']') {	/* Got a default answer for this item */
 		for (k = i = len; i && line[i] != '['; i--);
-		strncpy (answer, &line[i+1], k - i - 1);
+		strncpy (answer, &line[i+1], (size_t)(k - i - 1));
 	}
 	return (answer[0] != '\0');
 }
