@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: gmttrack.c,v 1.27 2007/10/07 07:10:41 guru Exp $
+ *	$Id: gmttrack.c,v 1.32 2008/04/28 21:27:37 guru Exp $
  *
- *    Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *    Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *    See README file for copying and redistribution conditions.
  *--------------------------------------------------------------------*/
 /*
@@ -48,16 +48,18 @@ int main (int argc, char **argv)
 {
 	double west = 0.0, east = 0.0, south = 0.0, north = 0.0, lat, lon, angle;
 	double last_lon = 0.0, last_lat = 0.0, annotsize = ANSIZE, factor = 0.001, dlon;
-	double dx, dy, x, y, x0, y0, dist = 0.0, start_dist = 0., stop_dist = 1.0E100; 
+	double dx, dy, x, y, x0, y0, dist = 0.0, start_dist = 0.0, stop_dist = 1.0E100; 
+	
+	int n_records, leg_year;
 	
 	int i, iw, ie, is, in, annot_dist = 0, tick_dist = 0, annot_time = 0, tick_time = 0;
-	int nfiles = 0, n_records, leg_year, rec, pen_type = 0, k, white[2], black[3];
+	int nfiles = 0, rec, pen_type = 0, k, white[3], black[3];
 	int error = FALSE, first, last = FALSE, annot_name = FALSE, both = FALSE, no_clip = FALSE;
 	int  annot_tick = FALSE, draw_tick = FALSE, start_time = 0, stop_time = 2000000000;
 	int this_julian, last_julian, year, month, day, hour, minute, second, n_alloc_c = GMT_SMALL_CHUNK;
-        int mon1, day1 = 0, year1, hour1, min1, mon2, day2 = 0, year2, hour2, min2, n_id = 0;
+	int mon1, day1 = 0, year1, hour1, min1, mon2, day2 = 0, year2, hour2, min2, n_id = 0;
         
-       	char agency[10], gmtfile[BUFSIZ], label[50], comment[BUFSIZ];
+	char agency[10], gmtfile[BUFSIZ], label[50], comment[BUFSIZ];
 	
 	struct LEG_ANOT *cruise_id = VNULL;
 	struct GMT_PEN pen;
@@ -301,7 +303,7 @@ int main (int argc, char **argv)
 						strcpy (cruise_id[n_id].text, argv[i]);
 						n_id++;
 						if (n_id == n_alloc_c) {
-							n_alloc_c += GMT_SMALL_CHUNK;
+							n_alloc_c <<= 1;
 							cruise_id = (struct LEG_ANOT *) GMT_memory ((void *)cruise_id, (size_t)n_alloc_c, sizeof (struct LEG_ANOT), "gmttrack");
 						}
 					}
@@ -403,6 +405,7 @@ int main (int argc, char **argv)
 	GMT_free ((void *)record);
 	GMT_free ((void *)distance);
 	
+	gmtmgg_end ();
 	GMT_end (argc, argv);
 
 	exit (EXIT_SUCCESS);
