@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: originator.c,v 1.36 2007/09/22 00:07:28 guru Exp $
+ *	$Id: originator.c,v 1.41 2008/03/24 08:58:33 guru Exp $
  *
- *   Copyright (c) 2000-2007 by P. Wessel
+ *   Copyright (c) 2000-2008 by P. Wessel
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -106,8 +106,9 @@ struct HOTSPOT {
 
 int main (int argc, char **argv)
 {
-	int i, j, k, n, kk, ns, nh, nc, np, x, y, n_max_spots, fno, n_files = 0, n_args, n_input = 5;
-	int n_fields, n_expected_fields, n_read, n_skipped = 0, n_best_hs = 1, n_out = 0, tdz_output = 0;
+	int x, y, n_max_spots, fno, n_files = 0, n_args, n_input = 5;
+	int n_fields, n_expected_fields, n_best_hs = 1, tdz_output = 0, n_out = 0;
+	GMT_LONG  i, j, k, n, kk, ns, nh, nc, np, n_read, n_skipped = 0;
 
 	BOOLEAN	error = FALSE, done, nofile = TRUE, truncate_ages = FALSE, first = TRUE, better;
 	BOOLEAN finite = FALSE, degree = FALSE;		/* fitite is TRUE if stage pole file contains finite rotation poles instead */
@@ -230,7 +231,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "usage: %s [<xyfiles>] -E<euler_file> -F<hotspot_file> [-C] [-D<d_km>]\n", GMT_program);
 		fprintf (stderr, "	[-H] [-L[flag]] [-N<upper_age>] [-Qr/t] [-S<n_hs>] [-T] [-V] [-W<maxdist>] [%s]\n\n", GMT_t_OPT);
 
-		if (GMT_give_synopsis_and_exit) exit (-1);
+		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
 		fprintf (stderr, "\txyfiles is one or more seamount (x,y,z,r,t) files\n");
 		fprintf (stderr, "\t-E Specify file name for Euler poles.\n");
@@ -279,14 +280,14 @@ int main (int argc, char **argv)
 
 	nh = hotspot_init (hsfile, &hotspot);
 	if (n_best_hs <= 0 || n_best_hs > nh) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR -S:  Give value between 1 and %d\n", GMT_program, nh);
+		fprintf (stderr, "%s: GMT SYNTAX ERROR -S:  Give value between 1 and %ld\n", GMT_program, nh);
 		exit (EXIT_FAILURE);
 	}
 	n_max_spots = MIN (n_best_hs, nh);
 
 	ns = spotter_init (efile, &p, TRUE, finite, FALSE, &upper_age, gmtdefs.verbose);
 
-	hot = (struct HOTSPOT *) GMT_memory (VNULL, nh, sizeof (struct HOTSPOT), GMT_program);
+	hot = (struct HOTSPOT *) GMT_memory (VNULL, (size_t)nh, sizeof (struct HOTSPOT), GMT_program);
 
 	x = (gmtdefs.xy_toggle[1]) ? 1 : 0;	y = 1 - x;		/* Set up which columns have x and y on output */
 
@@ -334,7 +335,7 @@ int main (int argc, char **argv)
 			if (GMT_io.status & GMT_IO_EOF) continue;
 
 			if (GMT_io.status & GMT_IO_MISMATCH) {
-				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d (skipped)\n", GMT_program, n_fields, n_expected_fields, n_read);
+				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld (skipped)\n", GMT_program, n_fields, n_expected_fields, n_read);
 				continue;
 			}
 			if (n_input == 3) {	/* set constant r,t values */
@@ -351,7 +352,7 @@ int main (int argc, char **argv)
 						t_smt = upper_age;
 					}
 					else {
-						fprintf (stderr, "%s: Seamounts near line %d has age (%g) > oldest stage (%g) (skipped)\n", GMT_program, n_read, t_smt, upper_age);
+						fprintf (stderr, "%s: Seamounts near line %ld has age (%g) > oldest stage (%g) (skipped)\n", GMT_program, n_read, t_smt, upper_age);
 						continue;
 					}
 				}
@@ -365,11 +366,11 @@ int main (int argc, char **argv)
 			z_smt = in[2];
 			r_smt = in[3];
 
-			if (gmtdefs.verbose && !(n % 10)) fprintf (stderr, "%s: Working on seamount # %5d\r", GMT_program, n);
+			if (gmtdefs.verbose && !(n % 10)) fprintf (stderr, "%s: Working on seamount # %5ld\r", GMT_program, n);
 
-			nc = spotter_forthtrack (&x_smt, &y_smt, &t_smt, 1, p, ns, d_km, 0.0, TRUE, NULL, &c);
+			nc = spotter_forthtrack (&x_smt, &y_smt, &t_smt, (GMT_LONG)1, p, ns, d_km, 0.0, TRUE, NULL, &c);
 
-			np = (int) c[0];
+			np = (GMT_LONG) c[0];
 
 			memcpy ((void *)hot, (void *)hotspot, nh * sizeof (struct HOTSPOT));
 
@@ -448,7 +449,7 @@ int main (int argc, char **argv)
 				if (hot[j].stage == 0) hot[j].stage++;
 			}
 
-			if (nh > 1) qsort ((void *)hot, nh, sizeof(struct HOTSPOT), comp_hs);
+			if (nh > 1) qsort ((void *)hot, (size_t)nh, sizeof(struct HOTSPOT), comp_hs);
 
 			if (hot[0].np_dist < max_dist) {
 				if (tdz_output == 1) {	/* Want time, dist, z output */
@@ -489,7 +490,7 @@ int main (int argc, char **argv)
 		if (fp != stdin) fclose (fp);
 	}
 
-	if (gmtdefs.verbose) fprintf (stderr, "%s: Working on seamount # %5d\n", GMT_program, n);
+	if (gmtdefs.verbose) fprintf (stderr, "%s: Working on seamount # %5ld\n", GMT_program, n);
 
 	GMT_free ((void *)hotspot);
 	GMT_free ((void *)hot);
@@ -533,7 +534,7 @@ int hotspot_init (char *file, struct HOTSPOT **p)
 		e[i].np_dist = 1.0e100;
 		i++;
 		if ((size_t)i == n_alloc) {
-			n_alloc += GMT_CHUNK;
+			n_alloc <<= 1;
 			e = (struct HOTSPOT *) GMT_memory ((void *)e, n_alloc, sizeof (struct HOTSPOT), GMT_program);
 		}
 	}
